# Load the data
nlsy_math_wide<-read.table("C:/GRE2016/Data/nlsy_math_wide_R.dat", na.string='.')

# Give the variable names
names(nlsy_math_wide)<-c('id','female','lb_wght','anti_k1',
                         'math2','math3','math4','math5','math6','math7','math8',
                         'age2','age3','age4','age5','age6','age7','age8',
                         'men2','men3','men4','men5','men6','men7','men8',
                         'spring2','spring3','spring4','spring5','spring6','spring7','spring8',
                         'anti2','anti3','anti4','anti5','anti6','anti7','anti8')

# Attach Data
#attach(nlsy_math_wide)

library(OpenMx)

## Typical Null Model ##
tnull.math.omx <- mxModel("Null Model", 
    type="RAM", mxData(observed=nlsy_math_wide, type="raw" ), 
    manifestVars=c("math2","math3","math4","math5","math6","math7","math8"), 

# variance and covariance paths 
mxPath(from=c("math2","math3","math4","math5","math6","math7","math8"), 
    arrows=2, free=TRUE, values=60),

# means and intercepts 
mxPath(from="one", to=c("math2","math3","math4","math5","math6","math7","math8"),  
    arrows=1, free=TRUE, values=40)

) # close model

tnull.math.fit = mxRun(tnull.math.omx)
summary(tnull.math.fit)

#free parameters:
#                name matrix   row   col  Estimate  Std.Error
#1  Null Model.S[1,1]      S math2 math2 105.45972  8.1474381
#2  Null Model.S[2,2]      S math3 math3 105.80348  7.2047184
#3  Null Model.S[3,3]      S math4 math4 103.08562  7.5014022
#4  Null Model.S[4,4]      S math5 math5  89.51163  6.5647205
#5  Null Model.S[5,5]      S math6 math6  98.07155  7.0241240
#6  Null Model.S[6,6]      S math7 math7 112.28251 12.0678130
#7  Null Model.S[7,7]      S math8 math8 132.01983 15.6803747
#8  Null Model.M[1,1]      M     1 math2  32.60893  0.5610752
#9  Null Model.M[1,2]      M     1 math3  39.88401  0.4954598
#10 Null Model.M[1,3]      M     1 math4  46.16936  0.5222180
#11 Null Model.M[1,4]      M     1 math5  49.77427  0.4905306
#12 Null Model.M[1,5]      M     1 math6  52.72315  0.5014628
#13 Null Model.M[1,6]      M     1 math7  55.35270  0.8056224
#14 Null Model.M[1,7]      M     1 math8  57.83104  0.9642422
#
#observed statistics:  2221 
#estimated parameters:  14 
#degrees of freedom:  2207 
#-2 log likelihood:  16595.24 
#number of observations:  933 


## Appropriate Null Model based on Widaman & Thompson (2003) ##
null.math.omx <- mxModel("Null Model", 
    type="RAM", mxData(observed=nlsy_math_wide, type="raw" ), 
    manifestVars=c("math2","math3","math4","math5","math6","math7","math8"), 

# variance and covariance paths 
mxPath(from=c("math2","math3","math4","math5","math6","math7","math8"), 
    arrows=2, free=TRUE, values=60, labels='theta'),

# means and intercepts 
mxPath(from="one", to=c("math2","math3","math4","math5","math6","math7","math8"),  
    arrows=1, free=TRUE, values=40, labels='tau')

) # close model

null.math.fit = mxRun(null.math.omx)
summary(null.math.fit)

#Summary of Null Model 
# 
#free parameters:
#   name matrix   row   col  Estimate Std.Error
#1 theta      S math2 math2 163.76414 4.9145088
#2   tau      M     1 math2  46.11975 0.2715411
#
#observed statistics:  2221 
#estimated parameters:  2 
#degrees of freedom:  2219 
#-2 log likelihood:  17626.53 
#number of observations:  933 

## Saturated Model ##
sat.math.omx <- mxModel("Saturated Model", 
    type="RAM", mxData(observed=nlsy_math_wide, type="raw" ), 
    manifestVars=c("math2","math3","math4","math5","math6","math7","math8"), 

# variance and covariance paths 
mxPath(from=c("math2","math3","math4","math5","math6","math7","math8"), 
    arrows=2, free=TRUE, connect='unique.pairs', 
    values = c(80, 0, 0, 0, 0, 0, 0,
                  80, 0, 0, 0, 0, 0,
                     80, 0, 0, 0, 0,
                        80, 0, 0, 0,
                           80, 0, 0,
                              80, 0,
                                 80)), 
# means and intercepts 
mxPath(from="one", to=c("math2","math3","math4","math5","math6","math7","math8"), 
    arrows=1, free=TRUE, values=40)

) # close model

sat.math.fit = mxRun(sat.math.omx)
summary(sat.math.fit)

#free parameters:
#                     name matrix   row   col  Estimate  Std.Error
#1  Saturated Model.S[1,1]      S math2 math2 100.40902  7.1201108
#2  Saturated Model.S[1,2]      S math2 math3  55.06413 10.3321098
#3  Saturated Model.S[2,2]      S math3 math3 108.47455  7.1675707
#4  Saturated Model.S[1,3]      S math2 math4  65.79509  5.8565456
#5  Saturated Model.S[2,3]      S math3 math4  53.80477  9.3447588
#6  Saturated Model.S[3,3]      S math4 math4  99.12535  6.4686416
#7  Saturated Model.S[1,4]      S math2 math5  53.39914  7.4687238
#8  Saturated Model.S[2,4]      S math3 math5  65.12066  5.7169198
#9  Saturated Model.S[3,4]      S math4 math5  61.47479  6.8904380
#10 Saturated Model.S[4,4]      S math5 math5  90.14625  6.2650364
#11 Saturated Model.S[1,5]      S math2 math6  62.26399  6.3213113
#12 Saturated Model.S[2,5]      S math3 math6  66.05118  6.5421143
#13 Saturated Model.S[3,5]      S math4 math6  76.07262  5.7834369
#14 Saturated Model.S[4,5]      S math5 math6  56.10745  9.3172220
#15 Saturated Model.S[5,5]      S math6 math6 102.73514  6.8545939
#16 Saturated Model.S[1,6]      S math2 math7  63.34386 10.0414430
#17 Saturated Model.S[2,6]      S math3 math7  77.89596  7.8356658
#18 Saturated Model.S[3,6]      S math4 math7  68.91515  7.4781241
#19 Saturated Model.S[4,6]      S math5 math7  74.46468  7.0993138
#20 Saturated Model.S[5,6]      S math6 math7  87.65082  8.6059564
#21 Saturated Model.S[6,6]      S math7 math7 115.44908 10.6703568
#22 Saturated Model.S[1,7]      S math2 math8  79.22930  9.4641502
#23 Saturated Model.S[2,7]      S math3 math8  68.78435  9.4340442
#24 Saturated Model.S[3,7]      S math4 math8  96.76771  8.4183170
#25 Saturated Model.S[4,7]      S math5 math8  76.37454  9.8283605
#26 Saturated Model.S[5,7]      S math6 math8  96.60798  8.3940888
#27 Saturated Model.S[6,7]      S math7 math8 113.75270 10.8939140
#28 Saturated Model.S[7,7]      S math8 math8 141.50456 13.5624861
#29 Saturated Model.M[1,1]      M     1 math2  32.60572  0.4851375
#30 Saturated Model.M[1,2]      M     1 math3  40.02477  0.4600391
#31 Saturated Model.M[1,3]      M     1 math4  45.78563  0.4402942
#32 Saturated Model.M[1,4]      M     1 math5  49.52236  0.4344019
#33 Saturated Model.M[1,5]      M     1 math6  52.64757  0.4279128
#34 Saturated Model.M[1,6]      M     1 math7  55.55046  0.5879910
#35 Saturated Model.M[1,7]      M     1 math8  57.28883  0.6802941
#
#observed statistics:  2221 
#estimated parameters:  35 
#degrees of freedom:  2186 
#-2 log likelihood:  15732.61 
#number of observations:  933 



### No Growth Model ###

ng.math.omx <- mxModel("No Growth, Path Specification", 
    type="RAM", mxData(observed=nlsy_math_wide, type="raw" ), 
    manifestVars=c("math2","math3","math4","math5","math6","math7","math8"), 
    latentVars="eta_1",

# variance paths 
mxPath(from=c("eta_1","math2","math3","math4","math5","math6","math7","math8"), 
    arrows=2, free=TRUE, values = c(80, 60, 60, 60, 60, 60, 60, 60), 
    labels=c("psi_11", "th", "th", "th", "th", "th", "th", "th")),

# factor loadings 
mxPath(from="eta_1", to=c("math2","math3","math4","math5","math6","math7","math8"), 
    arrows=1, free=FALSE, values=1),

# means and intercepts 
mxPath(from="one", to="eta_1", 
    arrows=1, free=TRUE, values=40, labels="alpha_1")

) # close model

ng.math.fit <- mxRun(ng.math.omx)

summary(ng.math.fit)

#free parameters:
#     name matrix   row   col  Estimate Std.Error lbound ubound
#1      th      S math2 math2 116.68281  4.547869              
#2  psi_11      S eta_1 eta_1  46.91614  4.832891              
#3 alpha_1      M     1 eta_1  45.91468  0.323980              
#
#observed statistics:  2221 
#estimated parameters:  3 
#degrees of freedom:  2218 
#-2 log likelihood:  17491.9 
#saturated -2 log likelihood:  NA 
#number of observations:  933 
#chi-square:  NA 
#p:  NA 
#Information Criteria: 
#     df Penalty Parameters Penalty Sample-Size Adjusted
#AIC:  13055.903           17497.90                   NA
#BIC:   2324.321           17512.42             17502.89
#CFI: NA 
#TLI: NA 
#RMSEA:  NA 
#timestamp: 2014-08-12 11:52:11 
#frontend time: 0.05000496 secs 
#backend time: 0.139014 secs 
#independent submodels time: 0 secs 
#wall clock time: 0.189019 secs 
#cpu time: 0.189019 secs 
#OpenMx version number: 999.0.0-3473 

#Saturated Model and Typical Null Model

summary(ng.math.fit, SaturatedLikelihood=15732.61, SaturatedDoF=2186, IndependenceLikelihood=tnull.math.fit, IndepdenceDoF=2207)

#free parameters:
#     name matrix   row   col  Estimate Std.Error
#1      th      S math2 math2 116.68222 4.5460911
#2  psi_11      S eta_1 eta_1  46.91675 4.8234093
#3 alpha_1      M     1 eta_1  45.91468 0.3239773
#
#observed statistics:  2221 
#estimated parameters:  3 
#degrees of freedom:  2218 
#-2 log likelihood:  17491.9 
#saturated -2 log likelihood:  15732.61 
#number of observations:  933 
#chi-square:  X2 ( df=32 ) = 1759.293,  p = 0
#Information Criteria: 
#      |  df Penalty  |  Parameters Penalty  |  Sample-Size Adjusted
#AIC:      13055.903               17497.90                       NA
#BIC:       2324.321               17512.42                 17502.89
#CFI: -1.052331 
#TLI: -0.3468423 
#RMSEA:  0.2405289  [95% CI (0.2292027, 0.2519906)]

#Saturated Model and Appropriate Null Model

summary(ng.math.fit, SaturatedLikelihood=15732.61, SaturatedDoF=2186, IndependenceLikelihood=null.math.fit, IndepdenceDoF=2219)

#free parameters:
#     name matrix   row   col  Estimate Std.Error
#1      th      S math2 math2 116.68222 4.5460911
#2  psi_11      S eta_1 eta_1  46.91675 4.8234093
#3 alpha_1      M     1 eta_1  45.91468 0.3239773
#
#observed statistics:  2221 
#estimated parameters:  3 
#degrees of freedom:  2218 
#-2 log likelihood:  17491.9 
#saturated -2 log likelihood:  15732.61 
#number of observations:  933 
#chi-square:  X2 ( df=32 ) = 1759.293,  p = 0
#Information Criteria: 
#      |  df Penalty  |  Parameters Penalty  |  Sample-Size Adjusted
#AIC:      13055.903               17497.90                       NA
#BIC:       2324.321               17512.42                 17502.89
#CFI: 0.07775469 
#TLI: 0.3947765 
#RMSEA:  0.2405289  [95% CI (0.2292027, 0.2519906)]


# Can also use these 

#summary(ng.math.fit, SaturatedLikelihood=sat.math.fit, IndependenceLikelihood=tnull.math.fit)
#ref=mxRefModels(ng.math.fit)
#summary(ng.math.fit, refModels=ref)


### Linear Growth Model ###

lg.math.omx <- mxModel("Linear Growth, Path Specification",
    type="RAM", mxData(observed=nlsy_math_wide, type="raw"), 
    manifestVars=c("math2","math3","math4","math5","math6","math7","math8"),
    latentVars=c("eta_1","eta_2"),

# residual variance paths 
mxPath(from=c("math2","math3","math4","math5","math6","math7","math8"),
    arrows=2, free=TRUE, values=60, labels="th"),

# latent variable variances and covariance paths 
mxPath(from=c("eta_1","eta_2"), arrows=2, connect="unique.pairs", 
    free=TRUE, values=c(1,0,1), labels=c("psi_11","psi_21","psi_22")),

# factor loadings 
mxPath(from="eta_1", to=c("math2","math3","math4","math5","math6","math7","math8"), 
    arrows=1, free=FALSE, values=1),

mxPath(from="eta_2", to=c("math2","math3","math4","math5","math6","math7","math8"), 
    arrows=1, free=FALSE, values=c(0, 1, 2, 3, 4, 5, 6)),

# means and intercepts 
mxPath(from="one", to=c("eta_1","eta_2"), 
    arrows=1, free=TRUE, values=c(40, 4), labels=c("alpha_1","alpha_2"))

) # close model

lg.math.fit <- mxRun(lg.math.omx)

summary(lg.math.fit, SaturatedLikelihood=sat.math.fit, IndependenceLikelihood=tnull.math.fit) 

#free parameters:
#     name matrix   row   col   Estimate  Std.Error lbound ubound
#1      th      S math2 math2 36.2295121 1.88201524              
#2  psi_11      S eta_1 eta_1 64.5621973 5.72696169              
#3  psi_21      S eta_1 eta_2 -0.1815431 1.17776670              
#4  psi_22      S eta_2 eta_2  0.7325821 0.33346285              
#5 alpha_1      M     1 eta_1 35.2674178 0.35541335              
#6 alpha_2      M     1 eta_2  4.3393314 0.08833084              
#
#observed statistics:  2221 
#estimated parameters:  6 
#degrees of freedom:  2215 
#-2 log likelihood:  15937.39 
#saturated -2 log likelihood:  NA 
#number of observations:  933 
#chi-square:  NA 
#p:  NA 
#Information Criteria: 
#     df Penalty Parameters Penalty Sample-Size Adjusted
#AIC: 11507.3857           15949.39                   NA
#BIC:   790.3181           15978.42             15959.36
#CFI: NA 
#TLI: NA 
#RMSEA:  NA 
#timestamp: 2014-08-12 12:40:38 
#frontend time: 0.03300309 secs 
#backend time: 0.158016 secs 
#independent submodels time: 0 secs 
#wall clock time: 0.1910191 secs 
#cpu time: 0.1910191 secs 
#OpenMx version number: 999.0.0-3473 